package org.eclipse.jdt.internal.compiler.lookup;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.FieldDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeParameter;
import org.eclipse.jdt.internal.compiler.ast.TypeReference;
import org.eclipse.jdt.internal.compiler.env.AccessRestriction;
import org.eclipse.jdt.internal.compiler.impl.ReferenceContext;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.CompilationUnitScope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.LocalTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.MemberTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
import org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.RawTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeCollisionException;
import org.eclipse.jdt.internal.compiler.lookup.Substitution;
import org.eclipse.jdt.internal.compiler.lookup.SyntheticMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;
import org.eclipse.jdt.internal.compiler.problem.AbortCompilation;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
import org.eclipse.jdt.internal.compiler.util.HashtableOfObject;

public class ClassScope extends Scope {
  public TypeDeclaration referenceContext;
  
  public TypeReference superTypeReference;
  
  ArrayList<Object> deferredBoundChecks;
  
  public ClassScope(Scope parent, TypeDeclaration context) {
    super(3, parent);
    this.referenceContext = context;
    this.deferredBoundChecks = null;
  }
  
  void buildAnonymousTypeBinding(SourceTypeBinding enclosingType, ReferenceBinding supertype) {
    LocalTypeBinding anonymousType = buildLocalType(enclosingType, enclosingType.fPackage);
    anonymousType.modifiers |= 0x8000000;
    int inheritedBits = supertype.typeBits;
    if ((inheritedBits & 0x4) != 0) {
      AbstractMethodDeclaration[] methods = this.referenceContext.methods;
      if (methods != null)
        for (int i = 0; i < methods.length; i++) {
          if (CharOperation.equals(TypeConstants.CLOSE, (methods[i]).selector) && (methods[i]).arguments == null) {
            inheritedBits &= 0x13;
            break;
          } 
        }  
    } 
    anonymousType.typeBits |= inheritedBits;
    if (supertype.isInterface()) {
      anonymousType.setSuperClass(getJavaLangObject());
      anonymousType.setSuperInterfaces(new ReferenceBinding[] { supertype });
      TypeReference typeReference = this.referenceContext.allocation.type;
      if (typeReference != null) {
        this.referenceContext.superInterfaces = new TypeReference[] { typeReference };
        if ((supertype.tagBits & 0x40000000L) != 0L) {
          problemReporter().superTypeCannotUseWildcard((SourceTypeBinding)anonymousType, typeReference, (TypeBinding)supertype);
          anonymousType.tagBits |= 0x20000L;
          anonymousType.setSuperInterfaces(Binding.NO_SUPERINTERFACES);
        } 
      } 
    } else {
      anonymousType.setSuperClass(supertype);
      anonymousType.setSuperInterfaces(Binding.NO_SUPERINTERFACES);
      TypeReference typeReference = this.referenceContext.allocation.type;
      if (typeReference != null) {
        this.referenceContext.superclass = typeReference;
        if ((supertype.erasure()).id == 41) {
          problemReporter().cannotExtendEnum((SourceTypeBinding)anonymousType, typeReference, (TypeBinding)supertype);
          anonymousType.tagBits |= 0x20000L;
          anonymousType.setSuperClass(getJavaLangObject());
        } else if (supertype.isFinal()) {
          problemReporter().anonymousClassCannotExtendFinalClass(typeReference, (TypeBinding)supertype);
          anonymousType.tagBits |= 0x20000L;
          anonymousType.setSuperClass(getJavaLangObject());
        } else if ((supertype.tagBits & 0x40000000L) != 0L) {
          problemReporter().superTypeCannotUseWildcard((SourceTypeBinding)anonymousType, typeReference, (TypeBinding)supertype);
          anonymousType.tagBits |= 0x20000L;
          anonymousType.setSuperClass(getJavaLangObject());
        } 
      } 
    } 
    connectMemberTypes();
    buildFieldsAndMethods();
    anonymousType.faultInTypesForFieldsAndMethods();
    anonymousType.verifyMethods(environment().methodVerifier());
  }
  
  void buildFields() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    if (sourceType.areFieldsInitialized())
      return; 
    if (this.referenceContext.fields == null) {
      sourceType.setFields(Binding.NO_FIELDS);
      return;
    } 
    FieldDeclaration[] fields = this.referenceContext.fields;
    int size = fields.length;
    int count = 0;
    for (int i = 0; i < size; i++) {
      switch (fields[i].getKind()) {
        case 1:
        case 3:
          count++;
          break;
      } 
    } 
    FieldBinding[] fieldBindings = new FieldBinding[count];
    HashtableOfObject knownFieldNames = new HashtableOfObject(count);
    count = 0;
    for (int j = 0; j < size; j++) {
      FieldDeclaration field = fields[j];
      if (field.getKind() != 2) {
        FieldBinding fieldBinding = new FieldBinding(field, null, field.modifiers | 0x2000000, (ReferenceBinding)sourceType);
        fieldBinding.id = count;
        checkAndSetModifiersForField(fieldBinding, field);
        if (knownFieldNames.containsKey(field.name)) {
          FieldBinding previousBinding = (FieldBinding)knownFieldNames.get(field.name);
          if (previousBinding != null)
            for (int f = 0; f < j; f++) {
              FieldDeclaration previousField = fields[f];
              if (previousField.binding == previousBinding) {
                problemReporter().duplicateFieldInType(sourceType, previousField);
                break;
              } 
            }  
          knownFieldNames.put(field.name, null);
          problemReporter().duplicateFieldInType(sourceType, field);
          field.binding = null;
        } else {
          knownFieldNames.put(field.name, fieldBinding);
          fieldBindings[count++] = fieldBinding;
        } 
      } 
    } 
    if (count != fieldBindings.length)
      System.arraycopy(fieldBindings, 0, fieldBindings = new FieldBinding[count], 0, count); 
    sourceType.tagBits &= 0xFFFFFFFFFFFFCFFFL;
    sourceType.setFields(fieldBindings);
  }
  
  void buildFieldsAndMethods() {
    buildFields();
    buildMethods();
    SourceTypeBinding sourceType = this.referenceContext.binding;
    if (!sourceType.isPrivate() && sourceType.superclass instanceof SourceTypeBinding && sourceType.superclass.isPrivate())
      ((SourceTypeBinding)sourceType.superclass).tagIndirectlyAccessibleMembers(); 
    if (sourceType.isMemberType() && !sourceType.isLocalType())
      ((MemberTypeBinding)sourceType).checkSyntheticArgsAndFields(); 
    ReferenceBinding[] memberTypes = sourceType.memberTypes;
    for (int i = 0, length = memberTypes.length; i < length; i++)
      ((SourceTypeBinding)memberTypes[i]).scope.buildFieldsAndMethods(); 
  }
  
  private LocalTypeBinding buildLocalType(SourceTypeBinding enclosingType, PackageBinding packageBinding) {
    this.referenceContext.scope = this;
    this.referenceContext.staticInitializerScope = new MethodScope(this, (ReferenceContext)this.referenceContext, true);
    this.referenceContext.initializerScope = new MethodScope(this, (ReferenceContext)this.referenceContext, false);
    LocalTypeBinding localType = new LocalTypeBinding(this, enclosingType, innermostSwitchCase());
    this.referenceContext.binding = (SourceTypeBinding)localType;
    checkAndSetModifiers();
    buildTypeVariables();
    ReferenceBinding[] memberTypeBindings = Binding.NO_MEMBER_TYPES;
    if (this.referenceContext.memberTypes != null) {
      int size = this.referenceContext.memberTypes.length;
      memberTypeBindings = new ReferenceBinding[size];
      int count = 0;
      for (int i = 0; i < size; i++) {
        LocalTypeBinding localTypeBinding;
        TypeDeclaration memberContext = this.referenceContext.memberTypes[i];
        switch (TypeDeclaration.kind(memberContext.modifiers)) {
          case 2:
          case 4:
            problemReporter().illegalLocalTypeDeclaration(memberContext);
            break;
          default:
            localTypeBinding = localType;
            while (true) {
              if (CharOperation.equals(((ReferenceBinding)localTypeBinding).sourceName, memberContext.name)) {
                problemReporter().typeCollidesWithEnclosingType(memberContext);
                break;
              } 
              ReferenceBinding referenceBinding = localTypeBinding.enclosingType();
              if (referenceBinding == null) {
                int j = 0;
                while (true) {
                  org.eclipse.jdt.internal.compiler.lookup.ClassScope memberScope;
                  if (j >= i) {
                    memberScope = new org.eclipse.jdt.internal.compiler.lookup.ClassScope(this, this.referenceContext.memberTypes[i]);
                    LocalTypeBinding memberBinding = memberScope.buildLocalType((SourceTypeBinding)localType, packageBinding);
                    memberBinding.setAsMemberType();
                    memberTypeBindings[count++] = (ReferenceBinding)memberBinding;
                    break;
                  } 
                  if (CharOperation.equals((this.referenceContext.memberTypes[memberScope]).name, memberContext.name)) {
                    problemReporter().duplicateNestedType(memberContext);
                    break;
                  } 
                  memberScope++;
                } 
                break;
              } 
            } 
            break;
        } 
      } 
      if (count != size)
        System.arraycopy(memberTypeBindings, 0, memberTypeBindings = new ReferenceBinding[count], 0, count); 
    } 
    localType.setMemberTypes(memberTypeBindings);
    return localType;
  }
  
  void buildLocalTypeBinding(SourceTypeBinding enclosingType) {
    LocalTypeBinding localType = buildLocalType(enclosingType, enclosingType.fPackage);
    connectTypeHierarchy();
    if ((compilerOptions()).sourceLevel >= 3211264L) {
      checkParameterizedTypeBounds();
      checkParameterizedSuperTypeCollisions();
    } 
    buildFieldsAndMethods();
    localType.faultInTypesForFieldsAndMethods();
    this.referenceContext.binding.verifyMethods(environment().methodVerifier());
  }
  
  private void buildMemberTypes(AccessRestriction accessRestriction) {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    ReferenceBinding[] memberTypeBindings = Binding.NO_MEMBER_TYPES;
    if (this.referenceContext.memberTypes != null) {
      int length = this.referenceContext.memberTypes.length;
      memberTypeBindings = new ReferenceBinding[length];
      int count = 0;
      for (int i = 0; i < length; i++) {
        SourceTypeBinding sourceTypeBinding;
        TypeDeclaration memberContext = this.referenceContext.memberTypes[i];
        if ((environment()).isProcessingAnnotations && environment().isMissingType(memberContext.name))
          throw new SourceTypeCollisionException(); 
        switch (TypeDeclaration.kind(memberContext.modifiers)) {
          case 2:
          case 4:
            if (sourceType.isNestedType() && 
              sourceType.isClass() && 
              !sourceType.isStatic()) {
              problemReporter().illegalLocalTypeDeclaration(memberContext);
              break;
            } 
          default:
            sourceTypeBinding = sourceType;
            while (true) {
              if (CharOperation.equals(((ReferenceBinding)sourceTypeBinding).sourceName, memberContext.name)) {
                problemReporter().typeCollidesWithEnclosingType(memberContext);
                break;
              } 
              ReferenceBinding referenceBinding = sourceTypeBinding.enclosingType();
              if (referenceBinding == null) {
                int j = 0;
                while (true) {
                  org.eclipse.jdt.internal.compiler.lookup.ClassScope memberScope;
                  if (j >= i) {
                    memberScope = new org.eclipse.jdt.internal.compiler.lookup.ClassScope(this, memberContext);
                    memberTypeBindings[count++] = (ReferenceBinding)memberScope.buildType(sourceType, sourceType.fPackage, accessRestriction);
                    break;
                  } 
                  if (CharOperation.equals((this.referenceContext.memberTypes[memberScope]).name, memberContext.name)) {
                    problemReporter().duplicateNestedType(memberContext);
                    break;
                  } 
                  memberScope++;
                } 
                break;
              } 
            } 
            break;
        } 
      } 
      if (count != length)
        System.arraycopy(memberTypeBindings, 0, memberTypeBindings = new ReferenceBinding[count], 0, count); 
    } 
    sourceType.setMemberTypes(memberTypeBindings);
  }
  
  void buildMethods() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    if (sourceType.areMethodsInitialized())
      return; 
    boolean isEnum = (TypeDeclaration.kind(this.referenceContext.modifiers) == 3);
    if (this.referenceContext.methods == null && !isEnum) {
      this.referenceContext.binding.setMethods(Binding.NO_METHODS);
      return;
    } 
    AbstractMethodDeclaration[] methods = this.referenceContext.methods;
    int size = (methods == null) ? 0 : methods.length;
    int clinitIndex = -1;
    for (int i = 0; i < size; i++) {
      if (methods[i].isClinit()) {
        clinitIndex = i;
        break;
      } 
    } 
    int count = isEnum ? 2 : 0;
    MethodBinding[] methodBindings = new MethodBinding[((clinitIndex == -1) ? size : (size - 1)) + count];
    if (isEnum) {
      methodBindings[0] = (MethodBinding)sourceType.addSyntheticEnumMethod(TypeConstants.VALUES);
      methodBindings[1] = (MethodBinding)sourceType.addSyntheticEnumMethod(TypeConstants.VALUEOF);
    } 
    boolean hasNativeMethods = false;
    if (sourceType.isAbstract()) {
      for (int j = 0; j < size; j++) {
        if (j != clinitIndex) {
          MethodScope scope = new MethodScope(this, (ReferenceContext)methods[j], false);
          MethodBinding methodBinding = scope.createMethod(methods[j]);
          if (methodBinding != null) {
            methodBindings[count++] = methodBinding;
            hasNativeMethods = !(!hasNativeMethods && !methodBinding.isNative());
          } 
        } 
      } 
    } else {
      boolean hasAbstractMethods = false;
      for (int j = 0; j < size; j++) {
        if (j != clinitIndex) {
          MethodScope scope = new MethodScope(this, (ReferenceContext)methods[j], false);
          MethodBinding methodBinding = scope.createMethod(methods[j]);
          if (methodBinding != null) {
            methodBindings[count++] = methodBinding;
            hasAbstractMethods = !(!hasAbstractMethods && !methodBinding.isAbstract());
            hasNativeMethods = !(!hasNativeMethods && !methodBinding.isNative());
          } 
        } 
      } 
      if (hasAbstractMethods)
        problemReporter().abstractMethodInConcreteClass(sourceType); 
    } 
    if (count != methodBindings.length)
      System.arraycopy(methodBindings, 0, methodBindings = new MethodBinding[count], 0, count); 
    sourceType.tagBits &= 0xFFFFFFFFFFFF3FFFL;
    sourceType.setMethods(methodBindings);
    if (hasNativeMethods) {
      for (int j = 0; j < methodBindings.length; j++)
        (methodBindings[j]).modifiers |= 0x8000000; 
      FieldBinding[] fields = sourceType.unResolvedFields();
      for (int k = 0; k < fields.length; k++)
        (fields[k]).modifiers |= 0x8000000; 
    } 
    if (isEnum && (compilerOptions()).isAnnotationBasedNullAnalysisEnabled) {
      LookupEnvironment environment = environment();
      ((SyntheticMethodBinding)methodBindings[0]).markNonNull(environment);
      ((SyntheticMethodBinding)methodBindings[1]).markNonNull(environment);
    } 
  }
  
  SourceTypeBinding buildType(SourceTypeBinding enclosingType, PackageBinding packageBinding, AccessRestriction accessRestriction) {
    this.referenceContext.scope = this;
    this.referenceContext.staticInitializerScope = new MethodScope(this, (ReferenceContext)this.referenceContext, true);
    this.referenceContext.initializerScope = new MethodScope(this, (ReferenceContext)this.referenceContext, false);
    if (enclosingType == null) {
      char[][] className = CharOperation.arrayConcat(packageBinding.compoundName, this.referenceContext.name);
      this.referenceContext.binding = new SourceTypeBinding(className, packageBinding, this);
    } else {
      char[][] className = CharOperation.deepCopy(enclosingType.compoundName);
      className[className.length - 1] = 
        CharOperation.concat(className[className.length - 1], this.referenceContext.name, '$');
      ReferenceBinding existingType = packageBinding.getType0(className[className.length - 1]);
      if (existingType != null && 
        !(existingType instanceof org.eclipse.jdt.internal.compiler.lookup.UnresolvedReferenceBinding))
        this.parent.problemReporter().duplicateNestedType(this.referenceContext); 
      this.referenceContext.binding = (SourceTypeBinding)new MemberTypeBinding(className, this, enclosingType);
    } 
    SourceTypeBinding sourceType = this.referenceContext.binding;
    environment().setAccessRestriction((ReferenceBinding)sourceType, accessRestriction);
    TypeParameter[] typeParameters = this.referenceContext.typeParameters;
    sourceType.typeVariables = (typeParameters == null || typeParameters.length == 0) ? Binding.NO_TYPE_VARIABLES : null;
    sourceType.fPackage.addType((ReferenceBinding)sourceType);
    checkAndSetModifiers();
    buildTypeVariables();
    buildMemberTypes(accessRestriction);
    return sourceType;
  }
  
  private void buildTypeVariables() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    TypeParameter[] typeParameters = this.referenceContext.typeParameters;
    if (typeParameters == null || typeParameters.length == 0) {
      sourceType.setTypeVariables(Binding.NO_TYPE_VARIABLES);
      return;
    } 
    sourceType.setTypeVariables(Binding.NO_TYPE_VARIABLES);
    if (sourceType.id == 1) {
      problemReporter().objectCannotBeGeneric(this.referenceContext);
      return;
    } 
    sourceType.setTypeVariables(createTypeVariables(typeParameters, (Binding)sourceType));
    sourceType.modifiers |= 0x40000000;
  }
  
  void resolveTypeParameter(TypeParameter typeParameter) {
    typeParameter.resolve(this);
  }
  
  private void checkAndSetModifiers() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    int modifiers = sourceType.modifiers;
    if ((modifiers & 0x400000) != 0)
      problemReporter().duplicateModifierForType(sourceType); 
    ReferenceBinding enclosingType = sourceType.enclosingType();
    boolean isMemberType = sourceType.isMemberType();
    if (isMemberType) {
      modifiers |= enclosingType.modifiers & 0x40000800;
      if (enclosingType.isInterface())
        modifiers |= 0x1; 
      if (sourceType.isEnum()) {
        if (!enclosingType.isStatic()) {
          problemReporter().nonStaticContextForEnumMemberType(sourceType);
        } else {
          modifiers |= 0x8;
        } 
      } else if (sourceType.isInterface()) {
        modifiers |= 0x8;
      } 
    } else if (sourceType.isLocalType()) {
      if (sourceType.isEnum()) {
        problemReporter().illegalLocalTypeDeclaration(this.referenceContext);
        sourceType.modifiers = 0;
        return;
      } 
      if (sourceType.isAnonymousType()) {
        modifiers |= 0x10;
        if (this.referenceContext.allocation.type == null)
          modifiers |= 0x4000; 
      } 
      Scope scope = this;
      do {
        MethodScope methodScope;
        MethodBinding method;
        switch (scope.kind) {
          case 2:
            methodScope = (MethodScope)scope;
            if (methodScope.isLambdaScope())
              methodScope = methodScope.namedMethodScope(); 
            if (methodScope.isInsideInitializer()) {
              SourceTypeBinding type = ((TypeDeclaration)methodScope.referenceContext).binding;
              if (methodScope.initializedField != null) {
                if (methodScope.initializedField.isViewedAsDeprecated() && !sourceType.isDeprecated())
                  modifiers |= 0x200000; 
                break;
              } 
              if (type.isStrictfp())
                modifiers |= 0x800; 
              if (type.isViewedAsDeprecated() && !sourceType.isDeprecated())
                modifiers |= 0x200000; 
              break;
            } 
            method = ((AbstractMethodDeclaration)methodScope.referenceContext).binding;
            if (method != null) {
              if (method.isStrictfp())
                modifiers |= 0x800; 
              if (method.isViewedAsDeprecated() && !sourceType.isDeprecated())
                modifiers |= 0x200000; 
            } 
            break;
          case 3:
            if (enclosingType.isStrictfp())
              modifiers |= 0x800; 
            if (enclosingType.isViewedAsDeprecated() && !sourceType.isDeprecated())
              modifiers |= 0x200000; 
            break;
        } 
        scope = scope.parent;
      } while (scope != null);
    } 
    int realModifiers = modifiers & 0xFFFF;
    if ((realModifiers & 0x200) != 0) {
      if (isMemberType) {
        if ((realModifiers & 0xFFFFD1F0) != 0)
          if ((realModifiers & 0x2000) != 0) {
            problemReporter().illegalModifierForAnnotationMemberType(sourceType);
          } else {
            problemReporter().illegalModifierForMemberInterface(sourceType);
          }  
      } else if ((realModifiers & 0xFFFFD1FE) != 0) {
        if ((realModifiers & 0x2000) != 0) {
          problemReporter().illegalModifierForAnnotationType(sourceType);
        } else {
          problemReporter().illegalModifierForInterface(sourceType);
        } 
      } 
      if (sourceType.sourceName == TypeConstants.PACKAGE_INFO_NAME && (compilerOptions()).targetJDK > 3211264L)
        modifiers |= 0x1000; 
      modifiers |= 0x400;
    } else if ((realModifiers & 0x4000) != 0) {
      if (isMemberType) {
        if ((realModifiers & 0xFFFFB7F0) != 0) {
          problemReporter().illegalModifierForMemberEnum(sourceType);
          modifiers &= 0xFFFFFBFF;
          realModifiers &= 0xFFFFFBFF;
        } 
      } else if (!sourceType.isLocalType()) {
        if ((realModifiers & 0xFFFFB7FE) != 0)
          problemReporter().illegalModifierForEnum(sourceType); 
      } 
      if (!sourceType.isAnonymousType()) {
        if ((this.referenceContext.bits & 0x800) != 0) {
          modifiers |= 0x400;
        } else {
          TypeDeclaration typeDeclaration1 = this.referenceContext;
          FieldDeclaration[] arrayOfFieldDeclaration = typeDeclaration1.fields;
          int fieldsLength = (arrayOfFieldDeclaration == null) ? 0 : arrayOfFieldDeclaration.length;
          if (fieldsLength != 0) {
            AbstractMethodDeclaration[] methods = typeDeclaration1.methods;
            int methodsLength = (methods == null) ? 0 : methods.length;
            boolean definesAbstractMethod = (typeDeclaration1.superInterfaces != null);
            for (int i = 0; i < methodsLength && !definesAbstractMethod; i++)
              definesAbstractMethod = methods[i].isAbstract(); 
            if (definesAbstractMethod) {
              boolean needAbstractBit = false;
              for (int j = 0; j < fieldsLength; j++) {
                FieldDeclaration fieldDecl = arrayOfFieldDeclaration[j];
                if (fieldDecl.getKind() == 3)
                  if (fieldDecl.initialization instanceof org.eclipse.jdt.internal.compiler.ast.QualifiedAllocationExpression) {
                    needAbstractBit = true;
                  } else {
                    // Byte code: goto -> 834
                  }  
              } 
              if (needAbstractBit)
                modifiers |= 0x400; 
            } 
          } 
        } 
        TypeDeclaration typeDeclaration = this.referenceContext;
        FieldDeclaration[] fields = typeDeclaration.fields;
        if (fields != null)
          for (int i = 0, fieldsLength = fields.length; i < fieldsLength; i++) {
            FieldDeclaration fieldDecl = fields[i];
            if (fieldDecl.getKind() == 3 && 
              fieldDecl.initialization instanceof org.eclipse.jdt.internal.compiler.ast.QualifiedAllocationExpression)
              // Byte code: goto -> 1000 
          }  
        modifiers |= 0x10;
      } 
    } else {
      if (isMemberType) {
        if ((realModifiers & 0xFFFFF3E0) != 0)
          problemReporter().illegalModifierForMemberClass(sourceType); 
      } else if (sourceType.isLocalType()) {
        if ((realModifiers & 0xFFFFF3EF) != 0)
          problemReporter().illegalModifierForLocalClass(sourceType); 
      } else if ((realModifiers & 0xFFFFF3EE) != 0) {
        problemReporter().illegalModifierForClass(sourceType);
      } 
      if ((realModifiers & 0x410) == 1040)
        problemReporter().illegalModifierCombinationFinalAbstractForClass(sourceType); 
    } 
    if (isMemberType) {
      if (enclosingType.isInterface()) {
        if ((realModifiers & 0x6) != 0) {
          problemReporter().illegalVisibilityModifierForInterfaceMemberType(sourceType);
          if ((realModifiers & 0x4) != 0)
            modifiers &= 0xFFFFFFFB; 
          if ((realModifiers & 0x2) != 0)
            modifiers &= 0xFFFFFFFD; 
        } 
      } else {
        int accessorBits = realModifiers & 0x7;
        if ((accessorBits & accessorBits - 1) > 1) {
          problemReporter().illegalVisibilityModifierCombinationForMemberType(sourceType);
          if ((accessorBits & 0x1) != 0) {
            if ((accessorBits & 0x4) != 0)
              modifiers &= 0xFFFFFFFB; 
            if ((accessorBits & 0x2) != 0)
              modifiers &= 0xFFFFFFFD; 
          } else if ((accessorBits & 0x4) != 0 && (accessorBits & 0x2) != 0) {
            modifiers &= 0xFFFFFFFD;
          } 
        } 
      } 
      if ((realModifiers & 0x8) == 0) {
        if (enclosingType.isInterface())
          modifiers |= 0x8; 
      } else if (!enclosingType.isStatic()) {
        problemReporter().illegalStaticModifierForMemberType(sourceType);
      } 
    } 
    sourceType.modifiers = modifiers;
  }
  
  private void checkAndSetModifiersForField(FieldBinding fieldBinding, FieldDeclaration fieldDecl) {
    int modifiers = fieldBinding.modifiers;
    ReferenceBinding declaringClass = fieldBinding.declaringClass;
    if ((modifiers & 0x400000) != 0)
      problemReporter().duplicateModifierForField(declaringClass, fieldDecl); 
    if (declaringClass.isInterface()) {
      modifiers |= 0x19;
      if ((modifiers & 0xFFFF) != 25)
        if ((declaringClass.modifiers & 0x2000) != 0) {
          problemReporter().illegalModifierForAnnotationField(fieldDecl);
        } else {
          problemReporter().illegalModifierForInterfaceField(fieldDecl);
        }  
      fieldBinding.modifiers = modifiers;
      return;
    } 
    if (fieldDecl.getKind() == 3) {
      if ((modifiers & 0xFFFF) != 0)
        problemReporter().illegalModifierForEnumConstant(declaringClass, fieldDecl); 
      fieldBinding.modifiers |= 0x8004019;
      return;
    } 
    int realModifiers = modifiers & 0xFFFF;
    if ((realModifiers & 0xFFFFFF20) != 0) {
      problemReporter().illegalModifierForField(declaringClass, fieldDecl);
      modifiers &= 0xFFFF00DF;
    } 
    int accessorBits = realModifiers & 0x7;
    if ((accessorBits & accessorBits - 1) > 1) {
      problemReporter().illegalVisibilityModifierCombinationForField(declaringClass, fieldDecl);
      if ((accessorBits & 0x1) != 0) {
        if ((accessorBits & 0x4) != 0)
          modifiers &= 0xFFFFFFFB; 
        if ((accessorBits & 0x2) != 0)
          modifiers &= 0xFFFFFFFD; 
      } else if ((accessorBits & 0x4) != 0 && (accessorBits & 0x2) != 0) {
        modifiers &= 0xFFFFFFFD;
      } 
    } 
    if ((realModifiers & 0x50) == 80)
      problemReporter().illegalModifierCombinationFinalVolatileForField(declaringClass, fieldDecl); 
    if (fieldDecl.initialization == null && (modifiers & 0x10) != 0)
      modifiers |= 0x4000000; 
    fieldBinding.modifiers = modifiers;
  }
  
  public void checkParameterizedSuperTypeCollisions() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    ReferenceBinding[] interfaces = sourceType.superInterfaces;
    Map<Object, Object> invocations = new HashMap<>(2);
    ReferenceBinding itsSuperclass = sourceType.isInterface() ? null : sourceType.superclass;
    for (int i = 0, length = interfaces.length; i < length; i++) {
      ReferenceBinding one = interfaces[i];
      if (one != null && (
        itsSuperclass == null || !hasErasedCandidatesCollisions((TypeBinding)itsSuperclass, (TypeBinding)one, invocations, (ReferenceBinding)sourceType, (ASTNode)this.referenceContext)))
        for (int k = 0; k < i; k++) {
          ReferenceBinding two = interfaces[k];
          if (two != null && 
            hasErasedCandidatesCollisions((TypeBinding)one, (TypeBinding)two, invocations, (ReferenceBinding)sourceType, (ASTNode)this.referenceContext))
            break; 
        }  
    } 
    TypeParameter[] typeParameters = this.referenceContext.typeParameters;
    for (int j = 0, paramLength = (typeParameters == null) ? 0 : typeParameters.length; j < paramLength; j++) {
      TypeParameter typeParameter = typeParameters[j];
      TypeVariableBinding typeVariable = typeParameter.binding;
      if (typeVariable != null && typeVariable.isValidBinding()) {
        TypeReference[] boundRefs = typeParameter.bounds;
        if (boundRefs != null) {
          boolean checkSuperclass = TypeBinding.equalsEquals(typeVariable.firstBound, (TypeBinding)typeVariable.superclass);
          int k, boundLength;
          label67: for (k = 0, boundLength = boundRefs.length; k < boundLength; k++) {
            TypeReference typeRef = boundRefs[k];
            TypeBinding superType = typeRef.resolvedType;
            if (superType != null && superType.isValidBinding()) {
              if (checkSuperclass && 
                hasErasedCandidatesCollisions(superType, (TypeBinding)typeVariable.superclass, invocations, (ReferenceBinding)typeVariable, (ASTNode)typeRef))
                break; 
              for (int index = typeVariable.superInterfaces.length; --index >= 0;) {
                if (hasErasedCandidatesCollisions(superType, (TypeBinding)typeVariable.superInterfaces[index], invocations, (ReferenceBinding)typeVariable, (ASTNode)typeRef))
                  break label67; 
              } 
            } 
          } 
        } 
      } 
    } 
    ReferenceBinding[] memberTypes = this.referenceContext.binding.memberTypes;
    if (memberTypes != null && memberTypes != Binding.NO_MEMBER_TYPES)
      for (int k = 0, size = memberTypes.length; k < size; k++)
        ((SourceTypeBinding)memberTypes[k]).scope.checkParameterizedSuperTypeCollisions();  
  }
  
  private void checkForInheritedMemberTypes(SourceTypeBinding sourceType) {
    SourceTypeBinding sourceTypeBinding2 = sourceType;
    ReferenceBinding[] interfacesToVisit = null;
    int nextPosition = 0;
    ReferenceBinding referenceBinding2;
    do {
      if (sourceTypeBinding2.hasMemberTypes())
        return; 
      ReferenceBinding[] itsInterfaces = sourceTypeBinding2.superInterfaces();
      if (itsInterfaces == null || itsInterfaces == Binding.NO_SUPERINTERFACES)
        continue; 
      if (interfacesToVisit == null) {
        interfacesToVisit = itsInterfaces;
        nextPosition = interfacesToVisit.length;
      } else {
        int itsLength = itsInterfaces.length;
        if (nextPosition + itsLength >= interfacesToVisit.length)
          System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition); 
        for (int a = 0; a < itsLength; a++) {
          ReferenceBinding next = itsInterfaces[a];
          int b = 0;
          while (true) {
            if (b >= nextPosition) {
              interfacesToVisit[nextPosition++] = next;
              break;
            } 
            if (TypeBinding.equalsEquals((TypeBinding)next, (TypeBinding)interfacesToVisit[b]))
              break; 
            b++;
          } 
        } 
      } 
    } while ((referenceBinding2 = sourceTypeBinding2.superclass()) != null && (referenceBinding2.tagBits & 0x10000L) == 0L);
    if (interfacesToVisit != null) {
      boolean needToTag = false;
      int i;
      for (i = 0; i < nextPosition; i++) {
        ReferenceBinding anInterface = interfacesToVisit[i];
        if ((anInterface.tagBits & 0x10000L) == 0L) {
          if (anInterface.hasMemberTypes())
            return; 
          needToTag = true;
          ReferenceBinding[] itsInterfaces = anInterface.superInterfaces();
          if (itsInterfaces != null && itsInterfaces != Binding.NO_SUPERINTERFACES) {
            int itsLength = itsInterfaces.length;
            if (nextPosition + itsLength >= interfacesToVisit.length)
              System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition); 
            for (int a = 0; a < itsLength; a++) {
              ReferenceBinding next = itsInterfaces[a];
              int b = 0;
              while (true) {
                if (b >= nextPosition) {
                  interfacesToVisit[nextPosition++] = next;
                  break;
                } 
                if (TypeBinding.equalsEquals((TypeBinding)next, (TypeBinding)interfacesToVisit[b]))
                  break; 
                b++;
              } 
            } 
          } 
        } 
      } 
      if (needToTag)
        for (i = 0; i < nextPosition; i++)
          (interfacesToVisit[i]).tagBits |= 0x10000L;  
    } 
    SourceTypeBinding sourceTypeBinding1 = sourceType;
    ReferenceBinding referenceBinding1;
    do {
      ((ReferenceBinding)sourceTypeBinding1).tagBits |= 0x10000L;
    } while ((referenceBinding1 = sourceTypeBinding1.superclass()) != null && (referenceBinding1.tagBits & 0x10000L) == 0L);
  }
  
  public void checkParameterizedTypeBounds() {
    for (int i = 0, l = (this.deferredBoundChecks == null) ? 0 : this.deferredBoundChecks.size(); i < l; i++) {
      Object toCheck = this.deferredBoundChecks.get(i);
      if (toCheck instanceof TypeReference) {
        ((TypeReference)toCheck).checkBounds(this);
      } else if (toCheck instanceof Runnable) {
        ((Runnable)toCheck).run();
      } 
    } 
    this.deferredBoundChecks = null;
    ReferenceBinding[] memberTypes = this.referenceContext.binding.memberTypes;
    if (memberTypes != null && memberTypes != Binding.NO_MEMBER_TYPES)
      for (int j = 0, size = memberTypes.length; j < size; j++)
        ((SourceTypeBinding)memberTypes[j]).scope.checkParameterizedTypeBounds();  
  }
  
  private void connectMemberTypes() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    ReferenceBinding[] memberTypes = sourceType.memberTypes;
    if (memberTypes != null && memberTypes != Binding.NO_MEMBER_TYPES)
      for (int i = 0, size = memberTypes.length; i < size; i++)
        ((SourceTypeBinding)memberTypes[i]).scope.connectTypeHierarchy();  
  }
  
  private boolean connectSuperclass() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    if (sourceType.id == 1) {
      sourceType.setSuperClass(null);
      sourceType.setSuperInterfaces(Binding.NO_SUPERINTERFACES);
      if (!sourceType.isClass())
        problemReporter().objectMustBeClass(sourceType); 
      if (this.referenceContext.superclass != null || (this.referenceContext.superInterfaces != null && this.referenceContext.superInterfaces.length > 0))
        problemReporter().objectCannotHaveSuperTypes(sourceType); 
      return true;
    } 
    if (this.referenceContext.superclass == null) {
      if (sourceType.isEnum() && (compilerOptions()).sourceLevel >= 3211264L)
        return connectEnumSuperclass(); 
      sourceType.setSuperClass(getJavaLangObject());
      return !detectHierarchyCycle(sourceType, sourceType.superclass, null);
    } 
    TypeReference superclassRef = this.referenceContext.superclass;
    ReferenceBinding superclass = findSupertype(superclassRef);
    if (superclass != null)
      if (!superclass.isClass() && (superclass.tagBits & 0x80L) == 0L) {
        problemReporter().superclassMustBeAClass(sourceType, superclassRef, superclass);
      } else if (superclass.isFinal()) {
        problemReporter().classExtendFinalClass(sourceType, superclassRef, (TypeBinding)superclass);
      } else if ((superclass.tagBits & 0x40000000L) != 0L) {
        problemReporter().superTypeCannotUseWildcard(sourceType, superclassRef, (TypeBinding)superclass);
      } else if ((superclass.erasure()).id == 41) {
        problemReporter().cannotExtendEnum(sourceType, superclassRef, (TypeBinding)superclass);
      } else {
        if ((superclass.tagBits & 0x20000L) != 0L || 
          !superclassRef.resolvedType.isValidBinding()) {
          sourceType.setSuperClass(superclass);
          sourceType.tagBits |= 0x20000L;
          return superclassRef.resolvedType.isValidBinding();
        } 
        sourceType.setSuperClass(superclass);
        sourceType.typeBits |= superclass.typeBits & 0x13;
        if ((sourceType.typeBits & 0x3) != 0)
          sourceType.typeBits |= sourceType.applyCloseableClassWhitelists(); 
        return true;
      }  
    sourceType.tagBits |= 0x20000L;
    sourceType.setSuperClass(getJavaLangObject());
    if ((sourceType.superclass.tagBits & 0x100L) == 0L)
      detectHierarchyCycle(sourceType, sourceType.superclass, null); 
    return false;
  }
  
  private boolean connectEnumSuperclass() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    ReferenceBinding rootEnumType = getJavaLangEnum();
    if ((rootEnumType.tagBits & 0x80L) != 0L) {
      sourceType.tagBits |= 0x20000L;
      sourceType.setSuperClass(rootEnumType);
      return false;
    } 
    boolean foundCycle = detectHierarchyCycle(sourceType, rootEnumType, null);
    TypeVariableBinding[] refTypeVariables = rootEnumType.typeVariables();
    if (refTypeVariables == Binding.NO_TYPE_VARIABLES) {
      problemReporter().nonGenericTypeCannotBeParameterized(0, null, (TypeBinding)rootEnumType, new TypeBinding[] { (TypeBinding)sourceType });
      return false;
    } 
    if (1 != refTypeVariables.length) {
      problemReporter().incorrectArityForParameterizedType(null, (TypeBinding)rootEnumType, new TypeBinding[] { (TypeBinding)sourceType });
      return false;
    } 
    ParameterizedTypeBinding superType = environment().createParameterizedType(
        rootEnumType, 
        new TypeBinding[] { environment().convertToRawType((TypeBinding)sourceType, false) }null);
    sourceType.tagBits |= superType.tagBits & 0x20000L;
    sourceType.setSuperClass((ReferenceBinding)superType);
    if (!refTypeVariables[0].boundCheck((Substitution)superType, (TypeBinding)sourceType, this, null).isOKbyJLS())
      problemReporter().typeMismatchError((TypeBinding)rootEnumType, refTypeVariables[0], (ReferenceBinding)sourceType, null); 
    return !foundCycle;
  }
  
  private boolean connectSuperInterfaces() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    sourceType.setSuperInterfaces(Binding.NO_SUPERINTERFACES);
    if (this.referenceContext.superInterfaces == null) {
      if (sourceType.isAnnotationType() && (compilerOptions()).sourceLevel >= 3211264L) {
        ReferenceBinding annotationType = getJavaLangAnnotationAnnotation();
        boolean foundCycle = detectHierarchyCycle(sourceType, annotationType, null);
        sourceType.setSuperInterfaces(new ReferenceBinding[] { annotationType });
        return !foundCycle;
      } 
      return true;
    } 
    if (sourceType.id == 1)
      return true; 
    boolean noProblems = true;
    int length = this.referenceContext.superInterfaces.length;
    ReferenceBinding[] interfaceBindings = new ReferenceBinding[length];
    int count = 0;
    for (int i = 0; i < length; i++) {
      TypeReference superInterfaceRef = this.referenceContext.superInterfaces[i];
      ReferenceBinding superInterface = findSupertype(superInterfaceRef);
      if (superInterface == null) {
        sourceType.tagBits |= 0x20000L;
        noProblems = false;
      } else {
        for (int j = 0; j < i; j++) {
          if (TypeBinding.equalsEquals((TypeBinding)interfaceBindings[j], (TypeBinding)superInterface)) {
            problemReporter().duplicateSuperinterface(sourceType, superInterfaceRef, superInterface);
            sourceType.tagBits |= 0x20000L;
            noProblems = false;
            // Byte code: goto -> 432
          } 
        } 
        if (!superInterface.isInterface() && (superInterface.tagBits & 0x80L) == 0L) {
          problemReporter().superinterfaceMustBeAnInterface(sourceType, superInterfaceRef, superInterface);
          sourceType.tagBits |= 0x20000L;
          noProblems = false;
        } else {
          if (superInterface.isAnnotationType())
            problemReporter().annotationTypeUsedAsSuperinterface(sourceType, superInterfaceRef, superInterface); 
          if ((superInterface.tagBits & 0x40000000L) != 0L) {
            problemReporter().superTypeCannotUseWildcard(sourceType, superInterfaceRef, (TypeBinding)superInterface);
            sourceType.tagBits |= 0x20000L;
            noProblems = false;
          } else {
            if ((superInterface.tagBits & 0x20000L) != 0L || 
              !superInterfaceRef.resolvedType.isValidBinding()) {
              sourceType.tagBits |= 0x20000L;
              noProblems &= superInterfaceRef.resolvedType.isValidBinding();
            } 
            sourceType.typeBits |= superInterface.typeBits & 0x13;
            if ((sourceType.typeBits & 0x3) != 0)
              sourceType.typeBits |= sourceType.applyCloseableInterfaceWhitelists(); 
            interfaceBindings[count++] = superInterface;
          } 
        } 
      } 
    } 
    if (count > 0) {
      if (count != length)
        System.arraycopy(interfaceBindings, 0, interfaceBindings = new ReferenceBinding[count], 0, count); 
      sourceType.setSuperInterfaces(interfaceBindings);
    } 
    return noProblems;
  }
  
  void connectTypeHierarchy() {
    SourceTypeBinding sourceType = this.referenceContext.binding;
    CompilationUnitScope compilationUnitScope = compilationUnitScope();
    boolean wasAlreadyConnecting = compilationUnitScope.connectingHierarchy;
    compilationUnitScope.connectingHierarchy = true;
    try {
      if ((sourceType.tagBits & 0x100L) == 0L) {
        sourceType.tagBits |= 0x100L;
        (environment()).typesBeingConnected.add(sourceType);
        boolean noProblems = connectSuperclass();
        noProblems &= connectSuperInterfaces();
        (environment()).typesBeingConnected.remove(sourceType);
        sourceType.tagBits |= 0x200L;
        noProblems &= connectTypeVariables(this.referenceContext.typeParameters, false);
        sourceType.tagBits |= 0x40000L;
        if (noProblems && sourceType.isHierarchyInconsistent())
          problemReporter().hierarchyHasProblems(sourceType); 
      } 
      connectMemberTypes();
    } finally {
      compilationUnitScope.connectingHierarchy = wasAlreadyConnecting;
    } 
    LookupEnvironment env = environment();
    try {
      env.missingClassFileLocation = this.referenceContext;
      checkForInheritedMemberTypes(sourceType);
    } catch (AbortCompilation e) {
      e.updateContext((ASTNode)this.referenceContext, (referenceCompilationUnit()).compilationResult);
      throw e;
    } finally {
      env.missingClassFileLocation = null;
    } 
  }
  
  public boolean deferCheck(Runnable check) {
    if ((compilationUnitScope()).connectingHierarchy) {
      if (this.deferredBoundChecks == null)
        this.deferredBoundChecks = new ArrayList(); 
      this.deferredBoundChecks.add(check);
      return true;
    } 
    return false;
  }
  
  private void connectTypeHierarchyWithoutMembers() {
    if (this.parent instanceof CompilationUnitScope) {
      if (((CompilationUnitScope)this.parent).imports == null)
        ((CompilationUnitScope)this.parent).checkAndSetImports(); 
    } else if (this.parent instanceof org.eclipse.jdt.internal.compiler.lookup.ClassScope) {
      ((org.eclipse.jdt.internal.compiler.lookup.ClassScope)this.parent).connectTypeHierarchyWithoutMembers();
    } 
    SourceTypeBinding sourceType = this.referenceContext.binding;
    if ((sourceType.tagBits & 0x100L) != 0L)
      return; 
    CompilationUnitScope compilationUnitScope = compilationUnitScope();
    boolean wasAlreadyConnecting = compilationUnitScope.connectingHierarchy;
    compilationUnitScope.connectingHierarchy = true;
    try {
      sourceType.tagBits |= 0x100L;
      (environment()).typesBeingConnected.add(sourceType);
      boolean noProblems = connectSuperclass();
      noProblems &= connectSuperInterfaces();
      (environment()).typesBeingConnected.remove(sourceType);
      sourceType.tagBits |= 0x200L;
      noProblems &= connectTypeVariables(this.referenceContext.typeParameters, false);
      sourceType.tagBits |= 0x40000L;
      if (noProblems && sourceType.isHierarchyInconsistent())
        problemReporter().hierarchyHasProblems(sourceType); 
    } finally {
      compilationUnitScope.connectingHierarchy = wasAlreadyConnecting;
    } 
  }
  
  public boolean detectHierarchyCycle(TypeBinding superType, TypeReference reference) {
    ReferenceBinding referenceBinding;
    if (!(superType instanceof ReferenceBinding))
      return false; 
    if (reference == this.superTypeReference) {
      if (superType.isTypeVariable())
        return false; 
      if (superType.isParameterizedType())
        referenceBinding = ((ParameterizedTypeBinding)superType).genericType(); 
      compilationUnitScope().recordSuperTypeReference((TypeBinding)referenceBinding);
      return detectHierarchyCycle(this.referenceContext.binding, referenceBinding, reference);
    } 
    if ((((TypeBinding)referenceBinding).tagBits & 0x100L) == 0L && referenceBinding instanceof SourceTypeBinding)
      ((SourceTypeBinding)referenceBinding).scope.connectTypeHierarchyWithoutMembers(); 
    return false;
  }
  
  private boolean detectHierarchyCycle(SourceTypeBinding sourceType, ReferenceBinding superType, TypeReference reference) {
    if (superType.isRawType())
      superType = ((RawTypeBinding)superType).genericType(); 
    if (TypeBinding.equalsEquals((TypeBinding)sourceType, (TypeBinding)superType)) {
      problemReporter().hierarchyCircularity(sourceType, superType, reference);
      sourceType.tagBits |= 0x20000L;
      return true;
    } 
    if (superType.isMemberType()) {
      ReferenceBinding current = superType.enclosingType();
      do {
        if (current.isHierarchyBeingActivelyConnected() && TypeBinding.equalsEquals((TypeBinding)current, (TypeBinding)sourceType)) {
          problemReporter().hierarchyCircularity(sourceType, current, reference);
          sourceType.tagBits |= 0x20000L;
          current.tagBits |= 0x20000L;
          return true;
        } 
      } while ((current = current.enclosingType()) != null);
    } 
    if (superType.isBinaryBinding()) {
      if (superType.problemId() != 1 && (superType.tagBits & 0x20000L) != 0L) {
        sourceType.tagBits |= 0x20000L;
        problemReporter().hierarchyHasProblems(sourceType);
        return true;
      } 
      boolean hasCycle = false;
      ReferenceBinding parentType = superType.superclass();
      if (parentType != null) {
        if (TypeBinding.equalsEquals((TypeBinding)sourceType, (TypeBinding)parentType)) {
          problemReporter().hierarchyCircularity(sourceType, superType, reference);
          sourceType.tagBits |= 0x20000L;
          superType.tagBits |= 0x20000L;
          return true;
        } 
        if (parentType.isParameterizedType())
          parentType = ((ParameterizedTypeBinding)parentType).genericType(); 
        hasCycle |= detectHierarchyCycle(sourceType, parentType, reference);
        if ((parentType.tagBits & 0x20000L) != 0L) {
          sourceType.tagBits |= 0x20000L;
          parentType.tagBits |= 0x20000L;
        } 
      } 
      ReferenceBinding[] itsInterfaces = superType.superInterfaces();
      if (itsInterfaces != null && itsInterfaces != Binding.NO_SUPERINTERFACES)
        for (int i = 0, length = itsInterfaces.length; i < length; i++) {
          ReferenceBinding anInterface = itsInterfaces[i];
          if (TypeBinding.equalsEquals((TypeBinding)sourceType, (TypeBinding)anInterface)) {
            problemReporter().hierarchyCircularity(sourceType, superType, reference);
            sourceType.tagBits |= 0x20000L;
            superType.tagBits |= 0x20000L;
            return true;
          } 
          if (anInterface.isParameterizedType())
            anInterface = ((ParameterizedTypeBinding)anInterface).genericType(); 
          hasCycle |= detectHierarchyCycle(sourceType, anInterface, reference);
          if ((anInterface.tagBits & 0x20000L) != 0L) {
            sourceType.tagBits |= 0x20000L;
            superType.tagBits |= 0x20000L;
          } 
        }  
      return hasCycle;
    } 
    if (superType.isHierarchyBeingActivelyConnected()) {
      TypeReference ref = ((SourceTypeBinding)superType).scope.superTypeReference;
      if (ref != null && ref.resolvedType != null && ((ReferenceBinding)ref.resolvedType).isHierarchyBeingActivelyConnected()) {
        problemReporter().hierarchyCircularity(sourceType, superType, reference);
        sourceType.tagBits |= 0x20000L;
        superType.tagBits |= 0x20000L;
        return true;
      } 
      if (ref != null && ref.resolvedType == null) {
        char[] referredName = ref.getLastToken();
        for (Iterator<SourceTypeBinding> iter = (environment()).typesBeingConnected.iterator(); iter.hasNext(); ) {
          SourceTypeBinding type = iter.next();
          if (CharOperation.equals(referredName, type.sourceName())) {
            problemReporter().hierarchyCircularity(sourceType, superType, reference);
            sourceType.tagBits |= 0x20000L;
            superType.tagBits |= 0x20000L;
            return true;
          } 
        } 
      } 
    } 
    if ((superType.tagBits & 0x100L) == 0L)
      ((SourceTypeBinding)superType).scope.connectTypeHierarchyWithoutMembers(); 
    if ((superType.tagBits & 0x20000L) != 0L)
      sourceType.tagBits |= 0x20000L; 
    return false;
  }
  
  private ReferenceBinding findSupertype(TypeReference typeReference) {
    CompilationUnitScope unitScope = compilationUnitScope();
    LookupEnvironment env = unitScope.environment;
    try {
      env.missingClassFileLocation = typeReference;
      typeReference.aboutToResolve(this);
      unitScope.recordQualifiedReference(typeReference.getTypeName());
      this.superTypeReference = typeReference;
      ReferenceBinding superType = (ReferenceBinding)typeReference.resolveSuperType(this);
      return superType;
    } catch (AbortCompilation e) {
      SourceTypeBinding sourceType = this.referenceContext.binding;
      if (sourceType.superInterfaces == null)
        sourceType.setSuperInterfaces(Binding.NO_SUPERINTERFACES); 
      e.updateContext((ASTNode)typeReference, (referenceCompilationUnit()).compilationResult);
      throw e;
    } finally {
      env.missingClassFileLocation = null;
      this.superTypeReference = null;
    } 
  }
  
  public ProblemReporter problemReporter() {
    MethodScope outerMethodScope;
    if ((outerMethodScope = outerMostMethodScope()) == null) {
      ProblemReporter problemReporter = (referenceCompilationUnit()).problemReporter;
      problemReporter.referenceContext = (ReferenceContext)this.referenceContext;
      return problemReporter;
    } 
    return outerMethodScope.problemReporter();
  }
  
  public TypeDeclaration referenceType() {
    return this.referenceContext;
  }
  
  public boolean hasDefaultNullnessFor(int location) {
    SourceTypeBinding binding = this.referenceContext.binding;
    if (binding != null) {
      int nullDefault = binding.getNullDefault();
      if (nullDefault != 0)
        return ((nullDefault & location) != 0); 
    } 
    return this.parent.hasDefaultNullnessFor(location);
  }
  
  public String toString() {
    if (this.referenceContext != null)
      return "--- Class Scope ---\n\n" + 
        this.referenceContext.binding.toString(); 
    return "--- Class Scope ---\n\n Binding not initialized";
  }
}
